<?php

namespace Drupal\simple_sitemap_pagination_url\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\simple_sitemap\Manager\Generator;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Controller for custom simple sitemap pages.
 */
class CustomSimpleSitemapController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The simple sitemap generator.
   *
   * @var \Drupal\simple_sitemap\Manager\Generator
   */
  protected $sitemapGenerator;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructs a CustomSimpleSitemapController object.
   *
   * @param \Drupal\simple_sitemap\Manager\Generator $sitemap_generator
   *   The simple sitemap generator.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(Generator $sitemap_generator, LoggerChannelFactoryInterface $logger_factory) {
    $this->sitemapGenerator = $sitemap_generator;
    $this->loggerFactory = $logger_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('simple_sitemap.generator'),
      $container->get('logger.factory')
    );
  }

  /**
   * Generates sitemap page.
   *
   * @param string $page
   *   The page number.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The sitemap response.
   */
  public function sitemapDynamicPage($page = 0) {
    $page_number = (int) $page;

    // Get the default sitemap variant.
    $sitemap_content = $this->getSitemapContent('default', $page_number);

    if (empty($sitemap_content)) {
      throw new NotFoundHttpException();
    }

    return $this->createSitemapResponse($sitemap_content);
  }

  /**
   * Generates sitemap page for specific variant.
   *
   * @param string $variant
   *   The sitemap variant.
   * @param string $page
   *   The page number.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The sitemap response.
   */
  public function sitemapVariantPage($variant, $page = '') {
    $page_number = empty($page) ? 0 : (int) $page;

    // Remove 'sitemap' prefix if present.
    $variant_name = str_replace('sitemap', '', $variant);
    if (empty($variant_name)) {
      $variant_name = 'default';
    }

    $sitemap_content = $this->getSitemapContent($variant_name, $page_number);

    if (empty($sitemap_content)) {
      throw new NotFoundHttpException();
    }

    return $this->createSitemapResponse($sitemap_content);
  }

  /**
   * Gets sitemap content for a specific variant and page.
   *
   * @param string $variant
   *   The sitemap variant.
   * @param int $page
   *   The page number.
   *
   * @return string
   *   The sitemap content.
   */
  protected function getSitemapContent($variant, $page) {
    try {
      // Get sitemap content from simple_sitemap.
      $sitemap = $this->sitemapGenerator
        ->setSitemaps($variant)
        ->getContent($page);

      return $sitemap;
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('simple_sitemap_pagination_url')->error('Error generating sitemap: @message', ['@message' => $e->getMessage()]);
      return '';
    }
  }

  /**
   * Creates a sitemap response.
   *
   * @param string $content
   *   The sitemap content.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response object.
   */
  protected function createSitemapResponse($content) {
    $response = new Response($content);
    $response->headers->set('Content-Type', 'application/xml; charset=utf-8');
    $response->headers->set('X-Robots-Tag', 'noindex');

    // Add caching headers.
    $response->setMaxAge(3600);
    $response->setPublic();

    return $response;
  }

}
