<?php

namespace Drupal\simple_sitemap_pagination_url\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Redirects old sitemap URLs to new format.
 */
class SitemapUrlSubscriber implements EventSubscriberInterface {

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a new LeadsForm.
   */
  public function __construct(RequestStack $requestStack) {
    $this->requestStack = $requestStack;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::REQUEST => ['onRequest', 100],
    ];
  }

  /**
   * Handles the request event.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The request event.
   */
  public function onRequest(RequestEvent $event) {
    $request = $event->getRequest();
    $path = $request->getPathInfo();

    // Only process GET requests.
    if ($request->getMethod() !== 'GET') {
      return;
    }

    // Handle main sitemap.xml with page parameter.
    if ($path === '/sitemap.xml' && $request->query->has('page')) {
      $page = (int) $request->query->get('page');
      $new_url = $this->buildNewSitemapUrl('', $page);

      $response = new RedirectResponse($new_url, 301);
      $event->setResponse($response);
      return;
    }

    // Redirect /sitemaps/{variant}/sitemap.xml?page=N.
    if (preg_match('#^/sitemaps/([^/]+)/sitemap\.xml$#', $path, $matches) && $request->query->has('page')) {
      $variant = $matches[1];
      $page = (int) $request->query->get('page');
      $response = new RedirectResponse($this->buildNewSitemapUrl($variant, $page), 301);
      $event->setResponse($response);
    }
  }

  /**
   * Builds the new sitemap URL format.
   *
   * @param string $variant
   *   The sitemap variant (empty for main sitemap).
   * @param int $page
   *   The page number.
   *
   * @return string
   *   The new URL.
   */
  protected function buildNewSitemapUrl($variant, $page) {
    $base_url = $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost();

    // For main sitemap (no variant)
    if (empty($variant)) {
      if ($page <= 0) {
        return $base_url . '/sitemap-{page}.xml';
      }
      return $base_url . '/sitemap-' . $page . '.xml';
    }

    // Return $base_url . '/' . $variant . 'sitemap' . $page . '.xml';.
    return $base_url . ($page === 0
      ? "/sitemaps/$variant/sitemap.xml"
      : "/sitemaps/$variant/sitemap-$page.xml");
  }

}
